use crate::botv2::{
    domain::meme::answer::{answer_meme, AnswerResult},
    init::{Context, Error},
};
use poise::CreateReply;
use tracing::instrument;

/// Answer keyword with a meme if it match a registered keyword
#[instrument(skip(ctx), level = "info", fields(channel = ctx.channel_id().get(), guild = ?ctx.guild_id().unwrap().get()))]
#[poise::command(slash_command, prefix_command, category = "meme")]
pub async fn answer(
    ctx: Context<'_>,
    #[description = "keyword thaat you want to search"] keyword: String,
) -> Result<(), Error> {
    let guild_id = match ctx.guild_id() {
        Some(guild) => guild.get(),
        None => return Ok(()),
    };
    let user_id = ctx.author().id.get();
    let config_img = ctx.data().config_img.clone();
    let base_path = ctx.data().config.image.path.clone();
    let bot_entity = ctx.data().entity_name.clone();
    let answer = answer_meme(
        user_id, guild_id, keyword, base_path, config_img, bot_entity,
    )
    .await;
    if let Ok(answer) = answer {
        match answer {
            AnswerResult::Embed(embeds, attachments) => {
                let reply = CreateReply {
                    embeds,
                    attachments,
                    ..Default::default()
                };
                if let Err(why) = ctx.send(reply).await {
                    tracing::error!("Error sending message: {:?}", why);
                }
            }
            AnswerResult::Animated(attachments) => {
                let reply = CreateReply {
                    attachments,
                    ..Default::default()
                };
                if let Err(why) = ctx.send(reply).await {
                    tracing::error!("Error sending message: {:?}", why);
                }
            }
        }
    } else {
        let builder = CreateReply::default().content("No meme keyword found");
        if let Err(why) = ctx.send(builder).await {
            tracing::error!("Error sending message: {:?}", why);
        }
    }
    Ok(())
}
