use crate::botv2::{
    domain::server_config::{
        admin_role::admin_role,
        check_if_server_enable_and_admin::check_if_server_enable_and_user_admin,
    },
    init::{Context, Error},
};
use poise::{
    serenity_prelude::{model::colour, CreateEmbed, CreateEmbedFooter, Role},
    CreateReply,
};
use tracing::instrument;

/// add/remove admin role
#[instrument(skip(ctx), level = "info", fields(channel = ctx.channel_id().get(), guild = ?ctx.guild_id().unwrap().get()))]
#[poise::command(
    slash_command,
    prefix_command,
    category = "server_config",
    guild_only = true
)]
pub async fn admin(
    ctx: Context<'_>,
    #[description = "Add admin role"] add: Option<Role>,
    #[description = "Remove admin role"] remove: Option<Role>,
) -> Result<(), Error> {
    let guild = match ctx.guild_id() {
        Some(guild) => guild,
        None => return Ok(()),
    };
    let entity_name = ctx.data().entity_name.clone();
    let footer = CreateEmbedFooter::new(entity_name.clone());
    match check_if_server_enable_and_user_admin(guild.get(), ctx.author().id.get(), ctx.http())
        .await
    {
        Ok((ok, _)) => {
            if !ok {
                let embed = CreateEmbed::new()
                    .title("You are not an admin")
                    .color(colour::Color::RED)
                    .footer(footer);
                if let Err(why) = ctx
                    .send(CreateReply::default().embed(embed).ephemeral(true))
                    .await
                {
                    tracing::error!("Error sending message: {:?}", why);
                }
                return Ok(());
            }
        }
        Err(_) => {
            let embed = CreateEmbed::new()
                .title("You are not an admin")
                .color(colour::Color::RED)
                .footer(footer);
            if let Err(why) = ctx
                .send(CreateReply::default().embed(embed).ephemeral(true))
                .await
            {
                tracing::error!("Error sending message: {:?}", why);
            }
            return Ok(());
        }
    };
    let role_add = match add {
        Some(role) => Some(role.id.get()),
        None => None,
    };
    let role_remove = match remove {
        Some(role) => Some(role.id.get()),
        None => None,
    };
    let mut output = match admin_role(guild.get(), role_add, role_remove).await {
        Ok(_) => CreateEmbed::new()
            .title("Success handling admin role")
            .color(colour::Color::DARK_GREEN),
        Err(_) => CreateEmbed::new()
            .title("Error handling admin role")
            .color(colour::Color::RED),
    };
    output = output.footer(footer);
    let mut builder = CreateReply::default().ephemeral(true);
    builder = builder.embed(output);
    if let Err(why) = ctx.send(builder).await {
        tracing::error!("Error sending message: {:?}", why);
    }
    Ok(())
}
