use serde::{Deserialize, Serialize};
use tracing::{info, instrument};

use crate::db::concour::Concour;

#[derive(Serialize, Deserialize, Debug, Clone)]
pub enum CreateConcourError {
    AlreadyExist,
    FindError(String),
    UnknownError(String),
}

#[instrument(level = "info")]
pub async fn create_concour(
    server_id: u64,
    channel_id: u64,
    title: String,
    description: String,
) -> Result<Option<Concour>, CreateConcourError> {
    let concour = match Concour::find_by_server_id_channel_id(&server_id, &channel_id).await {
        Ok(list_concour) => list_concour,
        Err(err) => {
            tracing::error!(error = err.to_string(), "Error finding concour");
            return Err(CreateConcourError::UnknownError(
                "Error finding concour".to_string(),
            ));
        }
    };

    if concour.is_some() {
        info!("Concour already exist");
        return Err(CreateConcourError::AlreadyExist);
    }

    let concour = Concour {
        server_id,
        channel_id,
        title,
        description,
        ..Default::default()
    };

    match concour.create().await {
        Ok(_) => {}
        Err(err) => {
            tracing::error!(error = err.to_string(), "Error creating concour");
            return Err(CreateConcourError::UnknownError(
                "Error creating concour".to_string(),
            ));
        }
    }
    Ok(Some(concour))
}
