use serde::{Deserialize, Serialize};
use tracing::{info, instrument};

use crate::db::concour::Concour;

#[derive(Serialize, Deserialize, Debug, Clone)]
pub enum ListConcourError {
    FindError(String),
    UnknownError(String),
}

#[instrument(level = "info")]
pub async fn list_concour(server_id: u64) -> Result<Vec<Concour>, ListConcourError> {
    let list_concour = match Concour::find_by_server_id(&server_id).await {
        Ok(list_concour) => list_concour,
        Err(err) => {
            tracing::error!(error = err.to_string(), "Error finding concour");
            return Err(ListConcourError::UnknownError(
                "Error finding concour".to_string(),
            ));
        }
    };
    if list_concour.is_empty() {
        info!("No concour found");
        return Err(ListConcourError::FindError("No concour found".to_string()));
    }

    Ok(list_concour)
}
